from univalidator.abstractions import BaseCompositeValidator, BaseValidator


class CompositeValidator[T](BaseCompositeValidator[T]):
    """Runs multiple validators on given data."""

    def add_validator(self, validator: BaseValidator[T]) -> None:
        """Add a new validator."""
        self.validators.append(validator)

    def validate(self, value: T) -> bool:
        """Return True if all validators pass."""
        is_valid: list[bool] = []
        errors: list[str] = []

        for validator in self.validators:
            if validator.validate(value):
                is_valid.append(True)
            else:
                is_valid.append(False)
                errors.append(validator.error)

        self.errors = errors
        return all(is_valid)



from pytest import mark

from univalidator.validators import MXEmailRecordValidator


def test_mxemail_record_validator_isinstance(
    mxemail: MXEmailRecordValidator[str],
) -> None:
    """Test mx email record validator isinstance or not."""
    assert isinstance(mxemail, MXEmailRecordValidator)


test_data: list[tuple[str, bool]] = [
    ("y646896@gmail.com", True),
    ("ios646896@gmail.com", True),
    ("shaileshpandit141@gmail.com", True),
    ("shaileshpandit141@dock.com", True),
    ("@domain.com", True),
    ("user@example.com", True),
    ("user@", False),
    ("user@domain", False),
    ("user@domain.", False),
    ("user@domain.c", False),
    ("eve@test-no-mx.invalid", False),
]


@mark.parametrize("value, expected", test_data)
def test_mxemail_record(
    mxemail: MXEmailRecordValidator[str],
    value: str,
    expected: bool,
) -> None:
    """Test mx email record."""
    assert mxemail.validate(value) == expected


test_data_allowed_domains: list[tuple[str, bool]] = [
    ("y646896@gmail.com", True),
    ("ios646896@gmail.com", True),
    ("shaileshpandit141@dock.com", False),
    ("shaileshpandit141@dock.com", False),
    ("shaileshpandit141@outlook.com", False),
    ("shaileshpandit141@outlook.com", False),
]


@mark.parametrize("value, expected", test_data_allowed_domains)
def test_mxemail_record_with_allowed_domains(
    mxemail_with_allowed_domains: MXEmailRecordValidator[str],
    value: str,
    expected: bool,
) -> None:
    """Test mx email record."""
    assert mxemail_with_allowed_domains.validate(value) == expected



from pytest import fixture

from univalidator.composites import CompositeValidator
from univalidator.validators import (
    MXEmailRecordValidator,
    RegexEmailValidator,
    RegexValidator,
)


@fixture
def username() -> RegexValidator[str]:
    """Create regex validator instance and return it."""
    return RegexValidator(
        pattern=r"^[A-Za-z][A-Za-z0-9._]{2,19}$",
    )


@fixture
def email() -> RegexEmailValidator[str]:
    """Create regex email validator instance and return it."""
    return RegexEmailValidator[str]()


@fixture
def mxemail() -> MXEmailRecordValidator[str]:
    """Create mx email record validator instance and return it."""
    return MXEmailRecordValidator[str]()


@fixture
def mxemail_with_allowed_domains() -> MXEmailRecordValidator[str]:
    """Create mx email record validator instance and return it."""
    return MXEmailRecordValidator[str](
        allowed_domains=["gmail.com"],
    )


@fixture
def composite_email() -> CompositeValidator[str]:
    "Run multiple validators."
    return CompositeValidator[str](
        validators=[
            RegexEmailValidator[str](),
            MXEmailRecordValidator[str](),
        ]
    )



from abc import ABC, abstractmethod


class BaseValidator[T](ABC):
    """Absctract class for email validator."""

    def __init__(
        self,
        *,
        error_message: str | None = None,
    ) -> None:
        """Initialize a base validator."""
        self.error_message = error_message or ""
        self.error: str = ""
        super().__init__()

    @abstractmethod
    def validate(self, value: T) -> bool:
        """This method use to validate email."""
        raise NotImplementedError



from pytest import mark

from univalidator.composites import CompositeValidator


def test_composite_validator_isinstance(
    composite_email: CompositeValidator[str],
) -> None:
    """Test composite validator isinstance or not."""
    assert isinstance(composite_email, CompositeValidator)


test_data: list[tuple[str, bool]] = [
    ("y646896@gmail.com", True),
    ("ios646896@gmail.com", True),
    ("shaileshpandit141@gmail.com", True),
    ("shaileshpandit141@dock.com", True),
    ("user@example.com", True),
    ("user@", False),
    ("user@domain", False),
    ("user@domain.", False),
    ("user@domain.c", False),
]


@mark.parametrize("value, expected", test_data)
def test_composite_email_address(
    composite_email: CompositeValidator[str],
    value: str,
    expected: bool,
) -> None:
    """Test valid email address."""
    assert composite_email.validate(value) == expected



from re import Match, match

from univalidator.abstractions import BaseValidator


class RegexValidator[T: str](BaseValidator[T]):
    """Regex base email validator."""

    def __init__(
        self,
        *,
        error_message: str | None = None,
        pattern: str,
    ) -> None:
        """Initialize a regex validator."""
        self.pattern = pattern
        super().__init__(error_message=error_message)

    def validate(self, value: T) -> bool:
        """Validate data by using regex."""
        matched: Match[str] | None = match(self.pattern, value)
        if matched is not None:
            return True

        self.error = self.error_message or "Value does not match the required format."
        return False



from pytest import mark

from univalidator.validators import RegexEmailValidator


def test_regex_email_validator_isinstance(
    email: RegexEmailValidator[str],
) -> None:
    """Test regex validator isinstance or not."""
    assert isinstance(email, RegexEmailValidator)


test_data: list[tuple[str, bool]] = [
    ("user@example.com", True),
    ("user.name@example.co", True),
    ("user-name@sub.domain.net", True),
    ("u@d.io", True),
    ("test_user@my-domain.org", True),
    ("user@", False),
    ("@domain.com", False),
    ("user@domain", False),
    ("user@domain.", False),
    ("user@@example.com", False),
    ("uuser name@example.com", False),
]


@mark.parametrize("value, expected", test_data)
def test_email_address(
    email: RegexEmailValidator[str],
    value: str,
    expected: bool,
) -> None:
    """Test valid email address."""
    assert email.validate(value) == expected



from ._mx_email_record_validator import MXEmailRecordValidator
from ._regex_email_validator import RegexEmailValidator
from ._regex_validator import RegexValidator

__all__ = [
    "MXEmailRecordValidator",
    "RegexEmailValidator",
    "RegexValidator",
]



import dns.resolver  # type: ignore

from univalidator.abstractions import BaseValidator


class MXEmailRecordValidator[T: str](BaseValidator[T]):
    """Validate email mx records"""

    def __init__(
        self,
        *,
        error_message: str | None = None,
        allowed_domains: list[str] | None = None,
    ) -> None:
        """Initialize a mc email record validator."""
        self.allowed_domains = allowed_domains
        super().__init__(error_message=error_message)

    def _has_mx_record(self, domain: str) -> bool:
        "Check email has mx record or not."
        try:
            records = dns.resolver.resolve(domain, "MX")  # type: ignore
            return len(records) > 0
        except dns.resolver.NoAnswer:
            self.error = f"No MX records found for domain '{domain}'."
        except dns.resolver.NXDOMAIN:
            self.error = f"Domain '{domain}' does not exist."
        except dns.resolver.Timeout:
            self.error = f"DNS lookup for '{domain}' timed out."
        except dns.resolver.NoNameservers:
            self.error = f"No nameservers available for '{domain}'."
        return False

    def validate(self, value: T) -> bool:
        """Validate email mx records."""
        domain = value.split("@")[-1]
        if self.allowed_domains:
            if domain in self.allowed_domains:
                return self._has_mx_record(domain)
            self.error = f"Domain {domain} is not allowed."
            return False
        return self._has_mx_record(domain)



from abc import ABC, abstractmethod

from ._base_validator import BaseValidator


class BaseCompositeValidator[T](ABC):
    """Base composite validator abstract class"""

    def __init__(
        self,
        *,
        validators: list[BaseValidator[T]],
    ) -> None:
        """Initialize a base ecomposite validator."""
        self.validators = validators
        self.errors: list[str] = []
        super().__init__()

    @abstractmethod
    def add_validator(self, validator: BaseValidator[T]) -> None:
        """Add a new validator."""
        raise NotImplementedError

    @abstractmethod
    def validate(self, value: T) -> bool:
        """Return True if all validators pass."""
        raise NotImplementedError



from univalidator.abstractions import BaseValidator

from ._regex_validator import RegexValidator


class RegexEmailValidator[T: str](BaseValidator[T]):
    """Regex base email validator."""

    def __init__(
        self,
        *,
        error_message: str | None = None,
        pattern: str = r"^[\w\.-]+@[\w\.-]+\.\w+$",
    ) -> None:
        """Initialize a regex email validator."""
        self.pattern = pattern
        super().__init__(error_message=error_message)

    def validate(self, value: T) -> bool:
        """Validate email by using regex."""
        validator = RegexValidator[T](
            error_message=self.error_message,
            pattern=self.pattern,
        )
        is_valid = validator.validate(value)
        if is_valid:
            return True

        self.error = validator.error
        return False



from pytest import mark

from univalidator.validators import RegexValidator


def test_regex_validator_isinstance(
    username: RegexValidator[str],
) -> None:
    """Test regex validator isinstance or not."""
    assert isinstance(username, RegexValidator)


test_data: list[tuple[str, bool]] = [
    ("Alice123", True),
    ("john_doe", True),
    ("Z9.test", True),
    ("Max_Power2025", True),
    ("a.b", True),
    ("Chris.underscore", True),
    ("s2.", True),
    ("1stPlace", False),
    ("_hidden", False),
    ("Al", False),
    ("Bob@home", False),
    ("toolooooooooooooooooong", False),
    ("24563", False),
    ("Space Man", False),
]


@mark.parametrize("value, expected", test_data)
def test_username(
    username: RegexValidator[str],
    value: str,
    expected: bool,
) -> None:
    """Test valid username"""
    assert username.validate(value) == expected
